#***********************************************************************************************************#
# Méthode de Monte-Carlo pour le calcul de la pente et de l'ordonnée à l'origine de la droite de regression #
# linéaire                                                                                                  #
# Programme publié par le GRIESP en 2019 - Modifications mineures apportées par Marie-Anne DEJOAN           #
#***********************************************************************************************************#

from scipy import *
from scipy.stats import linregress
import numpy as np
from matplotlib import pyplot as plt

# Renvoie une valeur aléatoire de la variable L[0] d'incertitude-type L[1]
def Alea(L):
    tirage=np.random.normal()   # Tirage entre -infini et +infini (loi normale)
    return L[0]+L[1]*tirage

# Entrées
m=[584.e-3,1.e-3]     # [valeur,incertitude-type] de chaque entrée
M=[58.44,0.01]
Vo=[0.50000,0.00025]
Vf=[50e-3,0.06e-3]    # Volume et incertitude de chaque solution fille (en L)

Vmere=[10e-3,15e-3,20e-3,30e-3,40e-3]     # Volume de solution mère prélevé pour préparer chaque solution fille (en L)
u_pipettes=[4e-5,5e-5,6e-5,7.2e-5,8.5e-5] # Incertitude associée au prélèvement de chaque volume de solution mère (en L)

cond=[0.2908,0.6421,0.8906,1.399,1.845]   # Conductivité mesurée pour chaque solution fille (en mS/cm)
InrelCond=0.02                            # Incertitude relative sur la mesure de la conductivité égale à 2%

# Préparation des listes avec incertitudes
sigma=[]
for k in range(len(cond)):
    sigma.append([cond[k],cond[k]*InrelCond])

Vm=[]
for k in range(len(Vmere)):
    Vm.append([Vmere[k],u_pipettes[k]])

# Méthode de Monte Carlo pour déterminer la pente et l'ordonnée à l'origine par régression linéaire
LCoef,LOrd=[],[] # Initialisation des listes vides (coefficient directeur et ordonnée à l'origine de la droite)
iteration=100000

for i in range(iteration):
    AleaCm=Alea(m)/(Alea(M)*Alea(Vo))
    AleaCf=[]
    AleaSigma=[]
    for k in range(len(Vmere)):
        AleaCf.append(AleaCm*Alea(Vm[k])/Alea(Vf))
        AleaSigma.append(Alea(sigma[k]))
    reg=linregress(AleaCf,AleaSigma)
    coef=reg[0]
    ord=reg[1]
    LCoef.append(coef)
    LOrd.append(ord)

MoyPente=mean(LCoef)
uPente=std(LCoef, ddof = 1)
MoyOrd=mean(LOrd)
uOrd=std(LOrd, ddof = 1)

print('Pente :',MoyPente,'mS.L.mol-1.cm-1')
print('u(Pente) :',uPente,'mS.L.mol-1.cm-1')
print('Ordonnee à l origine :',MoyOrd,'mS/cm')
print('u(Ordonnée à l origine) :',uOrd,'mS/cm')

fenetre1 = plt.figure('Pour 100000 iterations')
graph1 = fenetre1.add_subplot(121)
graph1.hist(LCoef, range=(min(LCoef),max(LCoef)), bins = 50, color = 'blue', edgecolor = 'black')
plt.xlabel('Pente (mS.L.mol-1.cm-1)')
plt.ylabel('Effectif')
graph2 = fenetre1.add_subplot(122)
graph2.hist(LOrd, range = (min(LOrd),max(LOrd)), bins = 50, color = 'purple', edgecolor = 'black')
plt.xlabel('Ordonnée à l origine (mS/cm)')
plt.ylabel('Effectif')

plt.show()

